#include "BSL_Commands_ROM.h"
#include "BSL_Command_Definitions_ROM.h"

dataBuffer TXBuffer;
dataBuffer RXBuffer;
unsigned int answer;

unsigned char (*BSL_RX_Packet)(dataBuffer *db);
unsigned char (*BSL_TX_Packet)(dataBuffer db);

/*******************************************************************************
*Function:    ROM_setMemOffset
*Description: sets offset to memory pointer for devices with more than 64kB of memory
*Parameters:
*             unsigned long int addr    value of the memory offset
*Returns:
*             unsigned char             BSL_ACK if operation was successfull
*******************************************************************************/
unsigned char ROM_setMemOffset(unsigned long int addr)
{
	TXBuffer.data[0] = SET_MEM_OFFSET;
	TXBuffer.data[1] = 0x04;
	TXBuffer.data[2] = 0x04;
	TXBuffer.data[3] = DUMMY_DATA;
	TXBuffer.data[4] = DUMMY_DATA;
	TXBuffer.data[5] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[6] = (unsigned char)((addr >> 8) & 0xFF);
	TXBuffer.size = 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer == ROM_ACK)
	{
		answer = BSL_ACK;
	}
	return answer;
}

/*******************************************************************************
*Function:    ROM_eraseCheck
*Description: verfies the erasure of flash memory within a certain address range
*Parameters:
*             unsigned long int addr        16-bit start address
*             unsigned long int lenngth     16-bit block length
*Returns:
*             unsigned char                 BSL_ACK if operation was successfull
*NOTE:
*             password protected
*******************************************************************************/
unsigned char ROM_eraseCheck(unsigned long int addr, unsigned long int length)
{
	TXBuffer.data[0] = ERASE_CHECK;
	TXBuffer.data[1] = 0x04;
	TXBuffer.data[2] = 0x04;
	TXBuffer.data[3] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[4] = (unsigned char)((addr >> 8) & 0xFF);
	TXBuffer.data[5] = (unsigned char)(length & 0xFF);
	TXBuffer.data[6] = (unsigned char)((length >> 8) & 0xFF);
	TXBuffer.size = 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer == ROM_ACK)
	{
		answer = BSL_ACK;
	}
	return answer;
}

/*******************************************************************************
*Function:    ROM_massErase
*Description: This command erases the entire flash memory area (main memory + info memory)
*Parameters:
*             none
*Returns:
*             unsigned char      BSL_ACK if operation was succesfull
*******************************************************************************/
unsigned char ROM_massErase(unsigned char family)
{
	TXBuffer.data[0] = MASS_ERASE;
	TXBuffer.data[1] = 0x04;
	TXBuffer.data[2] = 0x04;
	TXBuffer.data[3] = 0x00;
	TXBuffer.data[4] = 0xFF;
	TXBuffer.data[5] = 0x06;
	TXBuffer.data[6] = 0xA5;
	TXBuffer.size = 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer == ROM_ACK)
	{
		answer = BSL_ACK;
	}
	return answer; 
}

/*******************************************************************************
*Function:    ROM_eraseMainOrInfo
*Description: This command erases specific flash memory sections
*Parameters:
*             unsigned long int addr     start determines whether main or info memory is erased
*Returns:
*             unsigned char              BSL_ACK if operation was successfull
*NOTE:
*             password protected
*******************************************************************************/
unsigned char ROM_eraseMainOrInfo(unsigned long int addr)
{
	TXBuffer.data[0] = ERASE_MAIN_OR_INFO;
	TXBuffer.data[1] = 0x04;
	TXBuffer.data[2] = 0x04;
	TXBuffer.data[3] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[4] = (unsigned char)((addr >> 8) & 0xFF);
	TXBuffer.data[5] = 0x04;
	TXBuffer.data[6] = 0xA5;
	TXBuffer.size = 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer == ROM_ACK)
	{
		answer = BSL_ACK;
	}
	return answer;
}

/*******************************************************************************
*Function:    ROM_eraseSegment
*Description: This command erases specific flash memory segments
*Parameters:
*             unsigned long int addr    The address selects the appropriate segment.
*Returns:
*             unsigned char             BSL_ACK if operation was successfull
*NOTE:
*             password protected
*******************************************************************************/
unsigned char ROM_eraseSegment(unsigned long int addr)
{
	TXBuffer.data[0] = ERASE_SEGMENT;
	TXBuffer.data[1] = 0x04;
	TXBuffer.data[2] = 0x04;
	TXBuffer.data[3] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[4] = (unsigned char)((addr >> 8) & 0xFF);
	TXBuffer.data[5] = 0x02;
	TXBuffer.data[6] = 0xA5;
	TXBuffer.size = 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer == ROM_ACK)
	{
		answer = BSL_ACK;
	}
	return answer;
}

/*******************************************************************************
*Function:    ROM_TX_DataBlock
*Description: This command is used for any read access to flash memory, RAM or 
*             peripheral module control registers.
*Parameters:
*             unsigned long int addr        16-bit block start address
*             unsigned long int length      16-bit block length
*Returns:
*             unsigned char                 BSL_ACK if operation was successfull
*NOTE:
*             password protected
*******************************************************************************/
unsigned char ROM_TX_DataBlock(unsigned long int addr, unsigned long int length, unsigned char *buffer)
{
	TXBuffer.data[0] = TX_DATA_BLOCK;
	TXBuffer.data[1] = 0x04;
	TXBuffer.data[2] = 0x04;
	TXBuffer.data[3] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[4] = (unsigned char)((addr >> 8) & 0xFF);
	TXBuffer.data[5] = (unsigned char)length;
	TXBuffer.data[6] = 0;
	TXBuffer.size = 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer != BSL_HEADER)
	{
		return answer;
	}
	else
	{
		answer = BSL_RX_Packet(&RXBuffer);
		if (answer != ROM_ACK)
		{
			return answer;
		}
		else
		{
			return BSL_ACK;
		}
	}
}
/*******************************************************************************
*Function:    ROM_RX_DataBlock
*Description: This command is used for any write access to flash memory, RAM or
*             peripheral module control registers.
*Parameters:
*             DataBlock data        a struct containing data and address
*Returns:
*             unsigned char         BSL_ACK if operation was successfull
*NOTE:
*             password protected
*******************************************************************************/
unsigned char ROM_RX_DataBlock(DataBlock data)
{
	TXBuffer.data[0] = RX_DATA_BLOCK;
	TXBuffer.data[1] = (data.numberOfBytes + 4) & 0xFF;
	TXBuffer.data[2] = (data.numberOfBytes + 4) & 0xFF;
	TXBuffer.data[3] = (unsigned char)(data.startAddr & 0xFF);
	TXBuffer.data[4] = (unsigned char)((data.startAddr >> 8) & 0xFF);
	TXBuffer.data[5] = (data.numberOfBytes) & 0xFF;
	TXBuffer.data[6] = 0;
	unsigned int i;
	for (i = 0; i < data.numberOfBytes; i++)
	{
		TXBuffer.data[i + 7] = data.data[i];
	}
	TXBuffer.size = data.numberOfBytes + 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer == ROM_ACK)
	{
		answer = BSL_ACK;
	}
	return answer;
}

/*******************************************************************************
*Function:    ROM_RX_Password
*Description: This command is used to unlock the password-protected commands
*Parameters:
*             DataBlock data     a struct containing the password and dummy bytes
*Returns:
*             unsigned char      BSL_ACK if operation was successfull
*******************************************************************************/
unsigned char ROM_RX_Password(DataBlock data)
{
	TXBuffer.data[0] = RX_PASSWORD;
	TXBuffer.data[1] = 0x24;
	TXBuffer.data[2] = 0x24;
	TXBuffer.data[3] = DUMMY_DATA;
	TXBuffer.data[4] = DUMMY_DATA;
	TXBuffer.data[5] = DUMMY_DATA;
	TXBuffer.data[6] = DUMMY_DATA;
	unsigned int i;
	for (i = 0; i < data.numberOfBytes; i++)
	{
		TXBuffer.data[i + 7]=data.data[i];
	}
	TXBuffer.size = data.numberOfBytes + 7;
	answer = BSL_TX_Packet(TXBuffer);
	if ( answer == ROM_ACK )
	{
		answer = BSL_ACK;
	}
	return answer;
}

/*******************************************************************************
*Function:    ROM_get_RX_Buffer
*Description: returns RXBuffer
*Parameters:
*             none
*Returns:
*             dataBuffer RXBuffer
*******************************************************************************/
dataBuffer ROM_get_RX_Buffer()
{
	return RXBuffer;
}

/*******************************************************************************
*Function:    ROM_set_TX_Packet
*Description: sets TX_Packet
*Parameters:
*             unsigned char(*txFunc)(dataBuffer db)     containing the data
*Returns:
*             none
*******************************************************************************/
void ROM_set_TX_Packet(unsigned char (*txFunc)(dataBuffer  db))
{
	BSL_TX_Packet = txFunc;
}

/*******************************************************************************
*Function:    ROM_set_RX_Packet
*Description: sets RX_Packet
*Parameters:
*             unsigned char (*rxFunc)(dataBuffer *db)  pointer to save location
*Returns:
*            none
*******************************************************************************/
void ROM_set_RX_Packet(unsigned char (*rxFunc)(dataBuffer  *db))
{
	BSL_RX_Packet = rxFunc;
}